import mtop from '../mtop/index.alipay';
import { Device, } from '@alife/tg-iot-interface-api-device-list';
import IDevicePanelAPI, {
    DeviceControlOptions,
    GetDeviceStatusOptions,
    GetProductInfoOptions,
    GetProductTSLOptions,
    DeleteDeviceOptions,
    DeviceStatusChangeMessage,
    EMTHOD_ATTRIBUTE_SET,
    EMTHOD_SERVICE_INVOKE,
    MeshDeviceControlOptions,
} from '@alife/tg-iot-interface-api-device-panel'
import { APIError } from '@alife/tg-iot-interface-api-common';

const SkillIdMissingError: APIError = {
    code: 'internal_error',
    msg: 'skill_id 为空'
}

export class TMPSDK implements IDevicePanelAPI {

    private productInfoCache : {
        [key:string]: {
            skillId:number,
            [key:string]: any,
        };
    } = {};

    public controlDevice(request: DeviceControlOptions) {
        const productKey = request.productKey;
        const deviceId = request.deviceId;
        const method = request.method;
        const params = request.params;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        let skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        if (!method
            || (EMTHOD_ATTRIBUTE_SET !== method && EMTHOD_SERVICE_INVOKE !== method)) {
            throw new Error('method is invalid');
        }

        if (!params) {
            throw new Error('params is invalid');
        }

        this._handleSkillId(productKey, skillId, (skillId) => {

            // 非 515 走猫精接口
            if (515 != skillId) {
                mtop.sendMtop({
                    api: 'mtop.alibaba.aicloud.iot.deviceControlForAlipay',
                    v: '1.0',
                    needLogin: true,
                    data: {
                        devId: deviceId,
                        skillId: skillId,
                        params: JSON.stringify({
                            method,
                            params,
                            devId: deviceId,
                            "msgId": "",
                            "version": "1.0",
                        }),
                    },
                    success,
                    fail,
                    complete,
                });

                return;
            }

            // 515 为纯飞燕设备
            // 走弱生态链路， 接口文档参考:https://yuque.antfin-inc.com/ailabsfe/clpsv1/vaxx2g
            if(method === EMTHOD_ATTRIBUTE_SET) {
                mtop.sendMtop({
                    api: 'mtop.alibaba.aicloud.iot.aliyunH5ModuleForAlipay',
                    v: '1.0',
                    needLogin: true,
                    data: {
                        func: 'controlByApp',
                        args: JSON.stringify([
                            deviceId,
                            skillId,
                            JSON.stringify(params),
                        ]),
                    },
                    success,
                    fail,
                    complete,
                });
            } else if (method === EMTHOD_SERVICE_INVOKE) {
                mtop.sendMtop({
                    api: 'mtop.alibaba.aicloud.iot.aliyunH5ModuleForAlipay',
                    v: '1.0',
                    needLogin: true,
                    data: {
                        func: 'serviceInvoke',
                        args: JSON.stringify([{
                            deviceId,
                            skillId,
                            // @ts-ignore
                            identifier: params.identifier || '',
                            // @ts-ignore
                            args: JSON.stringify(params.args || {}),
                        }]),
                    },
                    success,
                    fail,
                    complete,
                });
            }

        }, (err) => {
            fail && fail(err);
            complete && complete(err);
        })();
    }

    public controlMeshDevice(request: MeshDeviceControlOptions) {
        throw new Error('Not Implemetated !');
    }

    public getDeviceConnectStatus(request: GetDeviceStatusOptions) {
        throw new Error('Not Implemetated !');
    }
    
    public getDeviceStatus(request: GetDeviceStatusOptions) {
        const productKey = request.productKey;
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        let skillId = request.skillId;

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this._handleSkillId(productKey, skillId, (skillId) => {
            mtop.sendMtop({
                api: 'mtop.alibaba.aicloud.iot.getDeviceStatusForAlipay',
                v: '1.0',
                needLogin: true,
                data: {
                    devId: deviceId,
                    skillId,
                },
                success,
                fail,
                complete,
            });
        }, (err) => {
            fail && fail(err);
            complete && complete(err);
        })();
    }

    public getProductInfo(request: GetProductInfoOptions) {
        const productKey = request.productKey;
        const skillId = request.skillId;
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;

        if (!productKey) {
            throw new Error('productKey is invalid');
        }

        if( skillId !== 515){
            mtop.sendMtop({
                api: 'mtop.alibaba.aicloud.iotopenplatformapp.getProductConfig',
                v: '1.0',
                data: {
                    productKey,
                },
                success,
                fail,
                complete,
            });
            return;
        }

        // skillId 为 515 时
        // 走弱生态链路， 接口文档参考:https://yuque.antfin-inc.com/ailabsfe/clpsv1/vaxx2g
        mtop.sendMtop({
            api: 'mtop.alibaba.aicloud.iot.aliyunH5ModuleForAlipay',
            v: '1.0',
            needLogin: true,
            data: {
                func: 'queryProductInfo',
                args: JSON.stringify([{
                    deviceId,
                    skillId:skillId.toString(),
                }]),
            },
            success,
            fail,
            complete,
        });
    }

    public getProductTSL(request: GetProductTSLOptions) {
        const productKey = request.productKey;
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        let skillId = request.skillId;

        if (!productKey) {
            throw new Error('productKey is invalid');
        }

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this._handleSkillId(productKey, skillId, (skillId) => {
            mtop.sendMtop({
                api: 'mtop.alibaba.aicloud.iot.getDeviceInfoForAlipay',
                v: '1.0',
                needLogin: true,
                data: {
                    productKey,
                    devId: deviceId,
                    skillId,
                },
                success,
                fail,
                complete,
            });
        }, (err) => {
            fail && fail(err);
            complete && complete(err);
        })();
    }

    public deleteDevice(request: DeleteDeviceOptions) {
        const productKey = request.productKey;
        const deviceId = request.deviceId;
        const success = request.success;
        const fail = request.fail;
        const complete = request.complete;
        let skillId = request.skillId;

        if (!productKey) {
            throw new Error('productKey is invalid');
        }

        if (!deviceId) {
            throw new Error('deviceId is invalid');
        }

        this._handleSkillId(productKey, skillId, (skillId) => {
            mtop.sendMtop({
                api: 'mtop.alibaba.aicloud.iot.deletedevice',
                v: '1.0',
                needLogin: true,
                data: {
                    unbindRequestJSONString: JSON.stringify({
                        productKey,
                        devId: deviceId,
                        skillId,
                    }),
                    deviceInfo: JSON.stringify({
                        "botId": 10,
                    }),
                },
                success,
                fail,
                complete,
            });
        }, (err) => {
            fail && fail(err);
            complete && complete(err);
        })();
    }

    log(msg) {
        console.log(msg)
    }

    /**
     * skill id 缺失时的补偿逻辑
     * @param deviceId 设备id
     * @param skillId 技能id
     * @param success 成功回调
     * @param fail 失败回调
     */
    private _handleSkillId(productKey:string, skillId:number, success, fail) {
        return () => {

            // 如果 skillId 未设置，尝试从缓存获取
            if (!skillId && this.productInfoCache[productKey]) {
                skillId = this.productInfoCache[productKey].skillId;
            }

            // 有 skillId, 触发成功回调
            if (skillId) {
                success && success(skillId);
                return;
            }

            if (!productKey) {
                fail && fail(new Error('productKey is invalid'));
            }

            // 拉取设备列表，以获取 skillId
            this.getProductInfo({
                productKey,
                success: (res) => {
                    this.productInfoCache[productKey] = res;
                    skillId = res.skillId;
                    success && success(skillId);
                },
                fail: (err) => { 
                    fail && fail(err);
                },
            });
        };
    }

    /**
     * 空实现，等子类覆盖
     * @param listener 
     */
    public onDeviceStatusChange(listener: (mesageArray: Array<DeviceStatusChangeMessage>) => void) {
    }

    /**
     * 空实现，等子类覆盖
     * @param listener 
     */
    public offDeviceStatusChange(listener: (mesageArray: Array<DeviceStatusChangeMessage>) => void) {
    }
};

export default new TMPSDK();