import { APIOptions, APIError, ERROR_CODE } from "@alife/tg-iot-interface-api-common"
import { EMTHOD_ATTRIBUTE_SET, DeviceStatusChangeMessage } from '@alife/tg-iot-interface-api-device-panel'
import { Device } from '@alife/tg-iot-interface-api-device-list';
import { DeviceListChangeMessage } from '@alife/tg-iot-interface-api-device-list';
import mtop from './mtop/index.rtos';

declare let my: any;

export class DeviceCache {
  [key: string]: Device;
};

const aiot = {
  _deviceCache: new DeviceCache(),
  sendMtop: mtop.sendMtop.bind(mtop),
};


// wrapp event "onDeviceInfoSync"
const deviceStatusChangelistenerList = new Array<Function>();
const onSyncDeviceInfo = (listener:Function)=>{
  if(deviceStatusChangelistenerList.indexOf(listener) != -1){
    console.warn('duplicated listener');
    return;
  }

  deviceStatusChangelistenerList.push(listener);
};

const offSyncDeviceInfo = (listener:Function)=>{
  const index = deviceStatusChangelistenerList.indexOf(listener);
  if(index == -1){
    console.warn('invalid listener');
    return;
  }

  deviceStatusChangelistenerList.splice(index, 1);
};

// wrapp event "onDeviceListChanged"
const deviceListChangelistenerList = new Array<Function>();
const onDeviceListChange = (listener: (DeviceListChangeMessage) => void) => {
  if(deviceListChangelistenerList.indexOf(listener) != -1){
    console.warn('duplicated listener');
    return;
  }

  deviceListChangelistenerList.push(listener);
};

const offDeviceListChange = (listener:Function)=>{
  const index = deviceListChangelistenerList.indexOf(listener);
  if(index == -1){
    console.warn('invalid listener');
    return;
  }

  deviceListChangelistenerList.splice(index, 1);
};

Object.assign(aiot, { onSyncDeviceInfo, offSyncDeviceInfo, onDeviceListChange, offDeviceListChange });

const API_LIST: Array<string> = [
  'getDeviceList',
  'controlDevice',
  'getDeviceStatus',
  'getProductInfo',
  'getProductTSL',
]

API_LIST.forEach(method => {
  aiot[method] = (options: APIOptions) => {
    let { success, fail, complete, deviceId, ...args } = options;

    // 为以下接口补充 skillId
    if (method === 'controlDevice'
      || method === 'getDeviceStatus'
      || method === 'getProductTSL') {
        const lowerCaseDeviceId = deviceId.toLowerCase();
        if (aiot._deviceCache[lowerCaseDeviceId]) {
            const device = aiot._deviceCache[lowerCaseDeviceId] as Device;
            const skillId = device.extra.skillId;
            const productKey = device.productKey;
            Object.assign(args, {
              skillId,
              productKey,
              devId: deviceId,
            });
        }
    }

    // 为控制接口拼接参数
    if(method === 'controlDevice'){
      const { skillId, method:controlMethod, params } = args;

      args = {
        devId: deviceId,
        skillId: skillId,
        params: JSON.stringify({
            method: controlMethod||EMTHOD_ATTRIBUTE_SET,
            params,
            devId: deviceId,
            "msgId": "",
            "version": "1.0",
        }),
      };
    }

    try {
      my.call(`aiot.${method}`, args, (res: any) => {
        const { data } = res;

        if(!data) {
            const apiError = {
                code: ERROR_CODE,
                msg: '无效的返回值'
              } as APIError;
            fail && fail(apiError);
            complete && complete(apiError);
            return;
        }

        const { success: isSuccess, model, msgCode, msgInfo } = data;
        if(isSuccess) {
            if(method === 'getDeviceList'){
              const { devices, zones } = model;
              aiot._deviceCache = new DeviceCache();
              devices.map(device => {
                  const dev = device as Device;
                  aiot._deviceCache[dev.devId.toLowerCase()] = dev;
              });
            }
            success(model);
            complete && complete(model);
          } else {
            const err = {
              code: msgCode,
              msg: msgInfo,
            } as APIError;
  
            fail && fail(err);
            complete && complete(res);
          }
      });
    } catch (err) {
      const apiError = {
        code: ERROR_CODE,
        msg: JSON.stringify(err),
      } as APIError;

      fail && fail(apiError);
      complete && complete(apiError);
    }
  }
});

const steupEventListener = () => {
  my.on('iot.onDeviceListChanged', (msg) => {
    console.log('>> iot.onDeviceListChanged for rtos', JSON.stringify(msg)); 
    const { data } = msg;
    
    if (!data) {
      console.warn('invalid data for message: iot.onDeviceListChanged'); 
      return;
    }

    // transfor map to array
    // like {"0":{"deviceId":"a80171ee000000667d6b07da78000000","powerstate":0}}}
    const DeviceListChangeMessageArray = Object.values(data) as Array<DeviceListChangeMessage>;
    
    if (!DeviceListChangeMessageArray) {
      console.warn('invalid DeviceListChangeMessageArray');
      return;
    }

    deviceListChangelistenerList.map(listener => {
      listener(DeviceListChangeMessageArray);
    });
  });

  my.on('iot.syncDeviceInfo', (msg:any) => {
    console.info('>> iot.syncDeviceInfo', JSON.stringify(msg)); 
    const { data } = msg;
    
    if (!data) {
      console.warn('invalid data for message: iot.onDeviceInfoSync'); 
      return;
    }

    // transfor map to array
    // like {"0":{"deviceId":"a80171ee000000667d6b07da78000000","powerstate":0}}}
    const syncDeviceInfoArray = Object.values(data) as Array<any>;
    const formatedAttrArray = new Array<DeviceStatusChangeMessage>();
    syncDeviceInfoArray.map(item => {
      const { deviceId, ...attrs } = item;
      formatedAttrArray.push({
        deviceId,
        attr: attrs,
      } as DeviceStatusChangeMessage);
    });
    
    if (!formatedAttrArray) {
      console.warn('invalid formatedAttrArray');
      return;
    }

    deviceStatusChangelistenerList.map(listener => {
      listener(formatedAttrArray);
    });
  });
};

export const steupEventListenerForRTOS = steupEventListener;

export default aiot;